//------------------------------------------------------
// module  : DisplayHDR.cc
// author  : Mignotte Max
// date    : 
// version : 1.0
// language: C++
// note    :
//------------------------------------------------------
//  

//------------------------------------------------
// INCLDED FILES ---------------------------------
//------------------------------------------------
#include "DisplayHDR.h"

//------------------------------------------------
// CONSTANTES ------------------------------------
//------------------------------------------------
const int ZOOM=1;
const int QUIT=0;
const int SAVE=0;
const int VISU=0; 

//------------------------------------------------
// VARIABLES GLOBALES ----------------------------
//------------------------------------------------
char Name_img[NCHAR];
char Extension[NCHAR];

float StGr;
int  NbIt;
float tau;

int  zoom;
int  flag_save;
int  flag_quit;
int  flag_visu;


//-------------------------
//--- Windows -------------
//-------------------------
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#include <X11/cursorfont.h>

Display   *display;
int	  screen_num;
int 	  depth;
Window	  root;
Visual*	  visual;
GC	  gc;

/************************************************************************/
/* OPEN_DISPLAY()							*/
/************************************************************************/
int open_display()
{
  if ((display=XOpenDisplay(NULL))==NULL)
   { printf("Connection impossible\n");
     return(-1); }

  else
   { screen_num=DefaultScreen(display);
     visual=DefaultVisual(display,screen_num);
     depth=DefaultDepth(display,screen_num);
     root=RootWindow(display,screen_num);
     return 0; }
}

/************************************************************************/
/* FABRIQUE_WINDOW()							*/
/* Cette fonction cre une fenetre X et l'affiche  l'cran.	        */
/************************************************************************/
Window fabrique_window(char *nom_fen,int x,int y,int width,int height,int zoom)
{
  Window                 win;
  XSizeHints      size_hints;
  XWMHints          wm_hints;
  XClassHint     class_hints;
  XTextProperty  windowName, iconName;

  char *name=nom_fen;

  if(zoom<0) { width/=-zoom; height/=-zoom; }
  if(zoom>0) { width*=zoom;  height*=zoom;  }

  win=XCreateSimpleWindow(display,root,x,y,width,height,1,0,255);

  size_hints.flags=PPosition|PSize|PMinSize;
  size_hints.min_width=width;
  size_hints.min_height=height;

  XStringListToTextProperty(&name,1,&windowName);
  XStringListToTextProperty(&name,1,&iconName);
  wm_hints.initial_state=NormalState;
  wm_hints.input=True;
  wm_hints.flags=StateHint|InputHint;
  class_hints.res_name=nom_fen;
  class_hints.res_class=nom_fen;

  XSetWMProperties(display,win,&windowName,&iconName,
                   NULL,0,&size_hints,&wm_hints,&class_hints);

  gc=XCreateGC(display,win,0,NULL);

  XSelectInput(display,win,ExposureMask|KeyPressMask|ButtonPressMask| 
               ButtonReleaseMask|ButtonMotionMask|PointerMotionHintMask| 
               StructureNotifyMask);

  XMapWindow(display,win);
  return(win);
}

/****************************************************************************/
/* CREE_XIMAGECOUL()							    */
/* Cre une XImage  partir d'un tableau 3 d d'entier                       */
/* L'image peut subir un zoom.						    */
/****************************************************************************/
XImage* cree_XimageCoul(int*** matRVB,int z,int length,int width)
{
  int i;
  int lgth,wdth,lig,col,zoom_col,zoom_lig;
  float somme;
  float sum[3];
  unsigned char	 pixR,pixV,pixB,pixN;
  unsigned char* dat;
  XImage* imageX;

  /*Zoom positif*/
  /*------------*/
  if (z>0)
  {
   lgth=length*z;
   wdth=width*z;

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<lgth;lig=lig+z) for(col=0;col<wdth;col=col+z)
   { 
    pixR=(unsigned char)matRVB[0][lig/z][col/z];
    pixV=(unsigned char)matRVB[1][lig/z][col/z];
    pixB=(unsigned char)matRVB[2][lig/z][col/z];
    somme=(1.0/3.0)*(pixR+pixV+pixB);
    pixN=(unsigned char)somme;

    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
      { 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+0)]=pixB; 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+1)]=pixV; 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+2)]=pixR; 
       dat[((lig+zoom_lig)*wdth*4)+((4*(col+zoom_col))+3)]=0; 
       }
    }
  } /*--------------------------------------------------------*/

  /*Zoom negatif*/
  /*------------*/
  else
  {
   z=-z;
   lgth=(length/z);
   wdth=(width/z);

   dat=(unsigned char*)malloc(lgth*(wdth*4)*sizeof(unsigned char));
   if (dat==NULL)
      { printf("Impossible d'allouer de la memoire.");
        exit(-1); }

  for(lig=0;lig<(lgth*z);lig=lig+z) for(col=0;col<(wdth*z);col=col+z)
   {  
    sum[0]=sum[1]=sum[2]=0.0;
    
    for(i=0;i<3;i++)
    for(zoom_lig=0;zoom_lig<z;zoom_lig++) for(zoom_col=0;zoom_col<z;zoom_col++)
     sum[i]+=matRVB[i][lig+zoom_lig][col+zoom_col];
       
    for(i=0;i<3;i++)  sum[i]/=(z*z); 

     dat[((lig/z)*wdth*4)+((4*(col/z))+0)]=(unsigned char)sum[1];
     dat[((lig/z)*wdth*4)+((4*(col/z))+1)]=(unsigned char)sum[1];
     dat[((lig/z)*wdth*4)+((4*(col/z))+2)]=(unsigned char)sum[1];
     dat[((lig/z)*wdth*4)+((4*(col/z))+3)]=(unsigned char)sum[1]; 
   }
  } /*--------------------------------------------------------*/

  imageX=XCreateImage(display,visual,depth,ZPixmap,0,(char*)dat,wdth,lgth,16,wdth*4);
  return (imageX);
}


//---------------------------------------------------------
//---------------------------------------------------------
// Enregistre les arguments de la ligne de commande      
//---------------------------------------------------------
//---------------------------------------------------------
void read_arguments(int argc,char** argv)
{
 int i;

   //Options
  if (argc<2) 
    { printf("\n Usage   : %s [Img. in hdr format]",argv[0]); 
      printf("\n Options : value by default indicated in ()");
      printf("\n            -----------------------------------------");
      printf("\n            -n NbIter ...........[%d]",NBITER); 
      printf("\n            -t StGrad ...........[%.5f]",STGR); 
      printf("\n            -a VarGradMax (%.4f)",TAU);
      printf("\n            -------------");
      printf("\n            -g Nom Extension");
      printf("\n            -------------");
      printf("\n            -s save (Sans)"); 
      printf("\n            -v visu (%d)",VISU);
      printf("\n            -q quit (Sans)"); 
      printf("\n            -z zoom (%d)",ZOOM);
      printf("\n            -------------\n");
      printf("\n WARNING !!");
      printf("  Check that the six first lines of the header file of your HDR image is in the following format:");
     

      printf("\n");
      printf("\n #?RADIANCE");
      printf("\n # Some comments");
      printf("\n EXPOSURE=  1.0000000000000");
      printf("\n FORMAT=32-bit_rle_rgbe");
      printf("\n");
      printf("\n -Y ysize +X xsize");
      printf("\n");
      printf("\n  Else, use an editor to modify the header file of your HDR image");
      printf("\n  To run the Program, type for example:");
      printf("\n  %s belgium.hdr -s",argv[0]);
      printf("\n\n\n\n\n");
      exit(-1); }

      //Chargement Fichier
      if (argc>1) strcpy(Name_img,argv[1]);
     

   //Boucle 
   for(i=2;i<argc;i++)
     {   
          switch(argv[i][1])
          {
           case 'n': NbIt=atoi(argv[++i]);            break; 
           case 't': StGr=atof(argv[++i]);            break; 
           case 'a': tau=atof(argv[++i]);             break; 
	   case 'g': strcpy(Extension,argv[++i]);     break;

           case 's': flag_save=1;                     break;
           case 'v': flag_visu=1;                     break;  
           case 'q': flag_quit=1;                     break;
           case 'z': zoom=atoi(argv[++i]);            break;                 
          }    
     } 
 }

//----------------------------------------------------------
//----------------------------------------------------------
// PROGRAMME PRINCIPAL -------------------------------------
//----------------------------------------------------------
//----------------------------------------------------------
int main(int argc,char** argv)
{
 int  flag;
 int lgth,wdth;

 //Pour Xwindow
 //------------
 XEvent ev;

 Window win_ppicture_imgResult;
 XImage *x_ppicture_imgResult;
 char nomfen_ppicture_imgResult[NCHAR];
 

 //Initialisation
 //--------------
 StGr=STGR;
 NbIt=NBITER;
 tau=TAU;
 
 zoom=ZOOM;
 flag_save=SAVE;
 flag_quit=QUIT;
 flag_visu=VISU;
 flag=0;

 //Lis Arguments
 //-------------
 read_arguments(argc,argv);
 
 //------------------------------------------
 //-------- Algorithme            -----------
 //------------------------------------------
 FILE* f=fopen(Name_img,"rb");
 RGBE_ReadHeader(f,&wdth,&lgth,NULL);
 DisplayHDR* dsplay=new DisplayHDR(lgth,wdth); 

  //>Options 
  strcpy((*dsplay).Name_Img,Name_img);
  strcpy((*dsplay).Extension,Extension);

  (*dsplay).StGr=StGr;
  (*dsplay).NbIter=NbIt;
  (*dsplay).Tau=tau;
  (*dsplay).flag_save=flag_save;
  (*dsplay).flag_visu=flag_visu; 

  //>Methode
  (*dsplay).Display_HDR();
  

//---------------------------------------------------------------
//---------------- visu sous XWINDOW ----------------------------
//---------------------------------------------------------------
 if (!flag_quit)
 {
 //ouverture session graphique
 if (open_display()<0) printf(" Impossible d'ouvrir une session graphique");

 //commentaire dans les photos affichees
 sprintf(nomfen_ppicture_imgResult,"ImageResult : %s",Name_img);
  
 //creation des fenetres
 win_ppicture_imgResult=fabrique_window(nomfen_ppicture_imgResult,10,10,(*dsplay).width,(*dsplay).length,zoom);

 //creation des Ximages
 x_ppicture_imgResult=cree_XimageCoul((*dsplay).ImgCoul,zoom,(*dsplay).length,(*dsplay).width);
 
 printf("\n\n Pour quitter,appuyer sur la barre d'espace");
 fflush(stdout);

 //boucle d'evenements
  for(;;)
     {
      XNextEvent(display,&ev);
       switch(ev.type)
        {
	 case Expose:   
          XPutImage(display,win_ppicture_imgResult,gc,x_ppicture_imgResult,0,0,0,0,
                   x_ppicture_imgResult->width,x_ppicture_imgResult->height);
                  
         break;

         case KeyPress: 
         XDestroyImage(x_ppicture_imgResult);
         
         XFreeGC(display,gc);
         XCloseDisplay(display);
         flag=1;
         break;
         }
   if (flag==1) break;
   }
 } 
       
//--------------- fin de la session graphique --------------     
//----------------------------------------------------------

   //retour sans probleme
   //--------------------
   cout << "\n C'est fini... \n" << endl ;
   exit(-1);
   return 0;
 }
 


