//---------------------------------------------------
// module  : DisplayHDR.h                         
// auteur  : Max Mignotte                            
// date    :                                              
// langage : C++                                      
// labo    :                                  
//---------------------------------------------------

#ifndef DISPLAYHDR_H
#define DISPLAYHDR_H

//------------------------------------------------
// FICHIERS INCLUS -------------------------------
//------------------------------------------------
#include "FunctionHDR.h"
#include "Chrono.h"
#include "rgbe.h"

//------------------------------------------------
// DEFINITIONS -----------------------------------
//------------------------------------------------
#define NEIGHB 3          //Model::DegreeNeigborhood (3)
#define TAU 0.005         //StoppingCriterion::VarMagnitudeGradient (0.005)
#define GRADMAX  0.8      //Model::ValGradMax (0.8)
#define MOYGAUSS 0.5      //InitGrad::Moy. Gaussienne (0.5) 
#define VARGAUSS 0.05     //InitGrad::Var. Gaussienne (0.05)
#define SATHDR 0.5        //ColorTreatment::Saturation (0.5)
#define PIXSAT 0.6        //ColorTreatment:%pixelsat (0.6)
#define NBBIN_IMG 256     //Nb Bins (256)
#define NBITER 10         //NbIterationsMax (10)
#define STGR  0.5         //PasGradient

#define FLAG_NAMEWITHCOM 1
#define FLAG_ALEATSORT   0

//------------------------------------------------
// CLASSE DISPLAYHDR -----------------------------
//------------------------------------------------
class DisplayHDR
{
  //--Donnees--
  //-----------
  public:

  //>Matrices Img
  int***   ImgCoul;
  float***  ImgCoulf;

  float**   Img1;
  float**   Img2;
  float**   Img3;
  float**   Img4;
  float**   Img5;
  float**   Img6;
  float**   Img7;
  float**   Img8;
  float**   Img9;

  //>Parametres <IMG>
  int width,length; 

  //>Parametre <ALGO>
  float StGr;       //StepGradient
  int   NbIter;     //NbIterations
  float Tau;        //StoppingCriterion::VarMagnitudeGradient (0.005)
  int   NewNeighb;  //Neighborhood

  //>Mesures Finales
  int   TimeSec;
  float NrjImg,NrjImgInit;  
  float VarMagnitude;
  float StGrInit;
  int   NbIterFinal;

  //>Flag
  int  flag_save;
  int  flag_visu;
 
  //>UtileInfo
  char Name_Img[NCHAR];
  char Extension[NCHAR];  
  char InfoProcess[100*NCHAR];
 
  public:
  DisplayHDR(int,int); 
  ~DisplayHDR();
       
  void Display_HDR();

 //===============================
 // HDR :: File>f_DisplayHDR.cc
 //===============================
 //>GradientOptimize
 void   HDR_ComputeConjGradDesc_(float**,int,int,int,int,float**,int,float**);
 void   HDR_ComputeNonLocalGraphWindow_(float**,int,int,float**,float*,int,int);
 void   HDR_ConjGradDesc_(float**,float**,int,int,float**,float*,int,int);
 void   HDR_DeriveNrj_(float**,float**,int,int,float**,float*,int);
 float  HDR_ComputeNrj_(float**,int,int,float**,float*,int);
 void   HDR_GivePosPixel_(int,int,int,int*,int*,int,int,int);
 
 //>StopCriterion
 float  HDR_ComputeVARGrad(float**,int,int);

 //>Luminance&ColorTreatment
 void  HDR_ConvertImgRGBToLum(float***,float**,int,int);
 void  HDR_RecalWithSat(float**,int,int,int,float);
 void  HDR_BuiltColorCompressedImg(float***,float**,float**,int,int,float***,float);
 void  HDR_BuiltColorCompressedImg_(float***,float**,float**,int,int,float***,float);

 //>HistoSpec
 float HDR_Histo(float);
 void  HDR_SpecHistoGradOnModel_M(float**,int,int,int);
 void  HDR_ExactSpecifGaussModel(float**,float**,int,int);
 void  HDR_ComputeHisto(float**,int,int);

 //Info
 void  HDR_VisuHisto(float**,int,int);
 void  HDR_FillInfoProcess();
 void  HDR_Stat(float**,int,int);
};

#endif 
